using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace NormalMapping
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        KeyboardState lastKbState;

        Model floor;
        Model walls;
        Model columns;

        Effect pointLight;
        Effect texturedLight;
        Effect normalMappedLight;

        Vector3 cameraPos = new Vector3(0, 2, 4.8f);
        Vector3 lightPosition;

        Texture2D marble;
        Texture2D tiles;

        Matrix view, projection;

        bool normalMappingOn = true;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            pointLight = Content.Load<Effect>("PointLight");
            texturedLight = Content.Load<Effect>("PointLightTextured");
            normalMappedLight = Content.Load<Effect>("PointLightNormalMap");

            floor = Content.Load<Model>("BathHouse-Floor");
            walls = Content.Load<Model>("BathHouse-Walls");
            columns = Content.Load<Model>("BathHouse-Cols");

            view = Matrix.CreateLookAt(cameraPos, cameraPos - Vector3.UnitZ, Vector3.Up);
            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(75), aspectRatio, 0.1f, 100);

            pointLight.Parameters["View"].SetValue(view);
            pointLight.Parameters["Projection"].SetValue(projection);

            pointLight.Parameters["cameraPosition"].SetValue(cameraPos);
            pointLight.Parameters["specularPower"].SetValue(50);
            pointLight.Parameters["specularIntensity"].SetValue(0);

            lightPosition = new Vector3(0, 2.5f, 0);
            pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            SetLightColors(pointLight);

            //--
            ChangeEffect(walls, pointLight);
            ChangeEffect(columns, texturedLight);
            ChangeEffect(floor, normalMappedLight);

            marble = Content.Load<Texture2D>("VeinedMarbleT");
            tiles = Content.Load<Texture2D>("ClassicTileT");
            texturedLight.Parameters["ModelTexture"].SetValue(marble);
            normalMappedLight.Parameters["ModelTexture"].SetValue(tiles);
        }

        private void SetLightColors(Effect effect)
        {
            effect.Parameters["ambientLightColor"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["diffuseLightColor"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
            effect.Parameters["specularLightColor"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbState = Keyboard.GetState();

            // Allows the game to exit
            if (kbState.IsKeyDown(Keys.Escape))
                this.Exit();


            if (kbState.IsKeyDown(Keys.Right))
            {
                lightPosition.X += 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }
            else if (kbState.IsKeyDown(Keys.Left))
            {
                lightPosition.X -= 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }
            if (kbState.IsKeyDown(Keys.Up))
            {
                lightPosition.Y += 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }
            else if (kbState.IsKeyDown(Keys.Down))
            {
                lightPosition.Y -= 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }
            if (kbState.IsKeyDown(Keys.PageDown))
            {
                lightPosition.Z += 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }
            else if (kbState.IsKeyDown(Keys.PageUp))
            {
                lightPosition.Z -= 0.1f;
                pointLight.Parameters["lightPosition"].SetValue(lightPosition);
            }

            if (kbState.IsKeyDown(Keys.Space) && lastKbState.IsKeyUp(Keys.Space))
            {
                normalMappingOn = !normalMappingOn;

                if (normalMappingOn)
                    ChangeEffect(floor, normalMappedLight);
                else
                    ChangeEffect(floor, texturedLight);
            }

            lastKbState = kbState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            DrawModel(walls, Matrix.Identity);
            texturedLight.Parameters["ModelTexture"].SetValue(marble);
            DrawModel(columns, Matrix.Identity);
            texturedLight.Parameters["ModelTexture"].SetValue(tiles);
            DrawModel(floor, Matrix.Identity);

            base.Draw(gameTime);
        }

        private void DrawModel(Model model, Matrix world)
        {
            Matrix[] transforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                    effect.Parameters["World"].SetValue(transforms[mesh.ParentBone.Index] * world);
                }
                mesh.Draw();
            }
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }
    }
}
